@@ -2,14 +2,12 @@ from __future__ import annotations
 
 import urllib
 from dataclasses import dataclass
-from typing import Optional, Sequence
+from typing import Any, Optional, Sequence
 
 import dateutil.parser
 from django.urls import reverse
 from django.utils import timezone
 
-from . import orm
-
 ASC, DSC = "asc", "dsc"
 
 
@@ -57,8 +55,7 @@ class Query:
                 path, lookup = path__lookup.rsplit("__", 1)
                 filters.append(QueryFilter(path, lookup, value))
 
-        res = cls(model_name, fields, filters)
-        return res
+        return cls(model_name, fields, filters)
 
     @property
     def field_str(self):
@@ -106,8 +103,6 @@ class MetaFieldType(type):
 
 
 class FieldType(metaclass=MetaFieldType):
-    default_value = ""
-
     def __init__(self):
         assert False
 
@@ -121,6 +116,7 @@ class FieldType(metaclass=MetaFieldType):
 
 
 class StringFieldType(FieldType):
+    default_value = ""
     lookups = {
         "equals": "string",
         "contains": "string",
@@ -134,6 +130,7 @@ class StringFieldType(FieldType):
         "not_regex": "string",
         "is_null": "boolean",
     }
+    aggregates = ["count"]
 
 
 class NumberFieldType(FieldType):
@@ -147,6 +144,11 @@ class NumberFieldType(FieldType):
         "lte": "number",
         "is_null": "boolean",
     }
+    aggregates = ["average", "count", "max", "min", "std_dev", "sum", "variance"]
+
+    @staticmethod
+    def format(value):
+        return float(value) if value is not None else None
 
     @staticmethod
     def parse(value):
@@ -154,6 +156,7 @@ class NumberFieldType(FieldType):
 
 
 class TimeFieldType(FieldType):
+    default_value = timezone.now().strftime("%Y-%m-%d %H:%M")
     lookups = {
         "equals": "time",
         "not_equals": "time",
@@ -163,7 +166,7 @@ class TimeFieldType(FieldType):
         "lte": "time",
         "is_null": "boolean",
     }
-    default_value = timezone.now().strftime("%Y-%m-%d %H:%M")
+    aggregates = ["count"]
 
     @staticmethod
     def parse(value):
@@ -171,16 +174,19 @@ class TimeFieldType(FieldType):
 
     @staticmethod
     def format(value):
-        return timezone.make_naive(value) if value else None
+        return str(timezone.make_naive(value)) if value else None
 
 
 class HTMLFieldType(FieldType):
+    default_value = None
     lookups = {}
+    aggregates = []
 
 
 class BooleanFieldType(FieldType):
     default_value = True
     lookups = {"equals": "boolean", "not_equals": "boolean", "is_null": "boolean"}
+    aggregates = ["average", "sum"]
 
     @staticmethod
     def parse(value):
@@ -205,24 +211,52 @@ TYPES = {
 }
 
 
+class BoundFieldMixin:
+    @property
+    def model_path(self):
+        return "__".join(self.path_parts)
+
+    @property
+    def field_path(self):
+        if self.model_path:
+            return f"{self.model_path}__{self.name}"
+        return self.name
+
+    @property
+    def path(self):
+        if self.aggregate:
+            return f"{self.field_path}__{self.aggregate}"
+        return self.field_path
+
+
 @dataclass
-class BoundFilter:
-    path: str
+class BoundFilter(BoundFieldMixin):
+    path_parts: Sequence[str]
+    name: str
+    aggregate: Optional[str]
     lookup: str
     value: str
     type_: FieldType
 
     @classmethod
-    def bind(cls, query_filter, orm_field):
+    def bind(cls, path, name, aggregate, query_filter, orm_field):
         return cls(
-            query_filter.path, query_filter.lookup, query_filter.value, orm_field.type_
+            path,
+            name,
+            aggregate,
+            query_filter.lookup,
+            query_filter.value,
+            orm_field.type_,
         )
 
     def __post_init__(self):
         self.parsed = None
         self.err_message = None
 
-        lookups = self.type_.lookups
+        if self.aggregate:
+            lookups = NumberFieldType.lookups
+        else:
+            lookups = self.type_.lookups
         if self.lookup not in lookups:
             self.err_message = f"Bad lookup '{self.lookup}' expected {lookups}"
         else:
@@ -235,51 +269,75 @@ class BoundFilter:
 
 
 @dataclass
-class BoundField:
-    path: str
+class BoundField(BoundFieldMixin):
+    path_parts: Sequence[str]
+    name: str
+    aggregate: Optional[str]
     direction: Optional[str]
     priority: Optional[int]
-    orm_field: orm.OrmField
+    orm_field: Any  # orm.OrmField
 
     def __post_init__(self):
         self.type_ = self.orm_field.type_
         self.concrete = self.orm_field.concrete
 
     @classmethod
-    def bind(cls, query_field, orm_field):
+    def bind(cls, path, name, aggregate, query_field, orm_field):
         direction = query_field.direction if orm_field.concrete else None
         priority = query_field.priority if orm_field.concrete else None
-        return cls(query_field.path, direction, priority, orm_field)
+        return cls(path, name, aggregate, direction, priority, orm_field)
 
 
 class BoundQuery:
     def __init__(self, query, orm_models):
         # orm_models = {model_name: {"fields": {field_name, FieldType}, "fks": {field_name: model_name}}}
-        def get_orm_field(path):
-            parts = path.split("__")
-            model_name = query.model_name
-            for part in parts[:-1]:
+        def get_path(parts, model_name):
+            for part in parts:
                 fk_field = orm_models[model_name].fks.get(part)
                 if fk_field is None:
                     return None
                 model_name = fk_field.model_name
-            res = orm_models[model_name].fields.get(parts[-1])
-            return res
+            return model_name
+
+        def get_orm_field(path):
+            parts = path.split("__")
+
+            # path__field
+            model_name = get_path(parts[:-1], query.model_name)
+            if model_name:
+                return (
+                    orm_models[model_name].fields.get(parts[-1]),
+                    parts[:-1],
+                    parts[-1],
+                    None,
+                )
+
+            # path__aggregate__field
+            model_name = get_path(parts[:-2], query.model_name)
+            if model_name:
+                orm_field = orm_models[model_name].fields.get(parts[-2])
+                if orm_field and parts[-1] in orm_field.type_.aggregates:
+                    return orm_field, parts[:-2], parts[-2], parts[-1]
+            return None, None, None, None
 
         self.model_name = query.model_name
         self.orm_models = orm_models
 
         self.fields = []
         for query_field in query.fields:
-            orm_field = get_orm_field(query_field.path)
+            orm_field, path, name, aggregate = get_orm_field(query_field.path)
             if orm_field:
-                self.fields.append(BoundField.bind(query_field, orm_field))
+                self.fields.append(
+                    BoundField.bind(path, name, aggregate, query_field, orm_field)
+                )
 
         self.filters = []
         for query_filter in query.filters:
-            orm_field = get_orm_field(query_filter.path)
+            orm_field, path, name, aggregate = get_orm_field(query_filter.path)
             if orm_field:
-                self.filters.append(BoundFilter.bind(query_filter, orm_field))
+                self.filters.append(
+                    BoundFilter.bind(path, name, aggregate, query_filter, orm_field)
+                )
 
     @property
     def sort_fields(self):
@@ -288,3 +346,7 @@ class BoundQuery:
     @property
     def calculated_fields(self):
         return {f.path for f in self.fields if not f.concrete}
+
+    @property
+    def valid_filters(self):
+        return [f for f in self.filters if f.is_valid]
