@@ -16,7 +16,7 @@ from django.views.decorators.csrf import csrf_exempt
 from . import version
 from .models import View
 from .orm import _OPEN_IN_ADMIN, get_models, get_results
-from .query import TYPES, BoundQuery, Query
+from .query import TYPES, BoundQuery, NumberFieldType, Query
 
 
 def _get_query_data(bound_query):
@@ -38,6 +38,35 @@ def _get_query_data(bound_query):
     }
 
 
+def _get_model_fields(orm_model):
+    def sort_model_fields(fields):
+        fields = sorted(fields)
+        front = {"id": 1, _OPEN_IN_ADMIN: 2}
+        return sorted(fields, key=lambda x: front.get(x, sys.maxsize))
+
+    def type_model(orm_field):
+        if orm_field.concrete and orm_field.type_.aggregates:
+            return orm_field.type_.name
+        else:
+            return None
+
+    fields = {
+        name: {
+            "model": type_model(orm_field),
+            "type": orm_field.type_.name,
+            "concrete": orm_field.concrete,
+        }
+        for name, orm_field in orm_model.fields.items()
+    }
+    fk_fields = {
+        name: {"model": fk_field.model_name, "type": None, "concrete": False}
+        for name, fk_field in orm_model.fks.items()
+    }
+    all_fields = {**fields, **fk_fields}
+
+    return {"fields": all_fields, "sortedFields": sort_model_fields(all_fields)}
+
+
 def _get_config(user, orm_models):
     types = {
         name: {
@@ -49,26 +78,22 @@ def _get_config(user, orm_models):
         for name, type_ in TYPES.items()
     }
 
-    def sort_model_fields(fields):
-        fields = sorted(fields)
-        front = {"id": 1, _OPEN_IN_ADMIN: 2}
-        return sorted(fields, key=lambda x: front.get(x, sys.maxsize))
-
     all_model_fields = {
-        model_name: {
-            "fields": {
-                name: {"type": orm_field.type_.name, "concrete": orm_field.concrete}
-                for name, orm_field in orm_model.fields.items()
-            },
-            "fks": {
-                name: {"model": fk_field.model_name}
-                for name, fk_field in orm_model.fks.items()
-            },
-            "sorted_fields": sort_model_fields(orm_model.fields),
-            "sorted_fks": sorted(orm_model.fks),
-        }
+        model_name: _get_model_fields(orm_model)
         for model_name, orm_model in orm_models.items()
     }
+    for type_ in TYPES.values():
+        all_model_fields[type_.name] = {
+            "fields": {
+                aggregate: {
+                    "type": NumberFieldType.name,
+                    "concrete": True,
+                    "model": None,
+                }
+                for aggregate in type_.aggregates
+            },
+            "sortedFields": type_.aggregates,
+        }
 
     saved_views = [
         {
@@ -90,7 +115,7 @@ def _get_config(user, orm_models):
         "adminUrl": admin_url,
         "types": types,
         "allModelFields": all_model_fields,
-        "sortedModels": sorted(all_model_fields),
+        "sortedModels": sorted(orm_models),
         "version": version,
         "savedViews": saved_views,
     }
@@ -110,7 +135,7 @@ def _get_context(request, model_name, fields):
 
 
 @admin_decorators.staff_member_required
-def query_ctx(request, *, model_name, fields=""):  # pragma: no cover
+def query_ctx(request, *, model_name, fields=""):
     ctx = _get_context(request, model_name, fields)
     return http.JsonResponse(ctx)
 
@@ -122,7 +147,11 @@ def query_html(request, *, model_name="", fields=""):
     ctx = ctx.replace("<", "\\u003C").replace(">", "\\u003E").replace("&", "\\u0026")
 
     if getattr(settings, "DATA_BROWSER_DEV", False):  # pragma: no cover
-        response = _get_from_js_dev_server(request)
+        try:
+            response = _get_from_js_dev_server(request)
+        except Exception as e:
+            return http.HttpResponse(f"Error loading from JS dev server.<br><br>{e}")
+
         template = engines["django"].from_string(response.text)
     else:
         template = loader.get_template("data_browser/index.html")
@@ -174,7 +203,7 @@ def _data_response(request, query, media, meta):
 def _get_from_js_dev_server(request):  # pragma: no cover
     import requests
 
-    upstream_url = f"http://localhost:3000{request.path}"
+    upstream_url = f"http://127.0.0.1:3000{request.path}"
     method = request.META["REQUEST_METHOD"].lower()
     return getattr(requests, method)(upstream_url, stream=True)
 
@@ -188,19 +217,9 @@ def proxy_js_dev_server(request, path):  # pragma: no cover
 
     """
     response = _get_from_js_dev_server(request)
-    content_type = response.headers.get("Content-Type")
-
-    if request.META.get("HTTP_UPGRADE", "").lower() == "websocket":
-        return http.HttpResponse(
-            content="WebSocket connections aren't supported",
-            status=501,
-            reason="Not Implemented",
-        )
-
-    else:
-        return http.StreamingHttpResponse(
-            streaming_content=response.iter_content(2 ** 12),
-            content_type=content_type,
-            status=response.status_code,
-            reason=response.reason,
-        )
+    return http.StreamingHttpResponse(
+        streaming_content=response.iter_content(2 ** 12),
+        content_type=response.headers.get("Content-Type"),
+        status=response.status_code,
+        reason=response.reason,
+    )
